@@ -1553,14 +1553,10 @@ void calcUminUmax(int ndata,
     (JRRoot+tid)->function = &JRStaeckelIntegrandSquared;
     (JRRoot+tid)->params = params+tid;
     //Find starting points for minimum
-    if ( fabs(GSL_FN_EVAL(JRRoot+tid,*(ux+ii))) < 0.0000001){ //we are at umin or umax
-      peps= GSL_FN_EVAL(JRRoot+tid,*(ux+ii)+0.000001);
-      meps= GSL_FN_EVAL(JRRoot+tid,*(ux+ii)-0.000001);
-      if ( fabs(peps) < 0.00000001 && fabs(meps) < 0.00000001 ) {//circular
-	*(umin+ii) = *(ux+ii);
-	*(umax+ii) = *(ux+ii);
-      }
-      else if ( peps < 0. && meps > 0. ) {//umax
+    peps= GSL_FN_EVAL(JRRoot+tid,*(ux+ii)+0.000001);
+    meps= GSL_FN_EVAL(JRRoot+tid,*(ux+ii)-0.000001);
+    if ( fabs(GSL_FN_EVAL(JRRoot+tid,*(ux+ii))) < 0.0000001 && peps*meps < 0. ){ //we are at umin or umax
+      if ( peps < 0. && meps > 0. ) {//umax
 	*(umax+ii)= *(ux+ii);
 	u_lo= 0.9 * (*(ux+ii) - 0.000001);
 	u_hi= *(ux+ii) - 0.0000001;
@@ -1595,7 +1591,7 @@ void calcUminUmax(int ndata,
 	  *(umin+ii) = gsl_root_fsolver_root ((s+tid)->s);
 	}
       }
-      else if ( peps > 0. && meps < 0. ){//umin
+      else {// JB: Should catch all: if ( peps > 0. && meps < 0. ){//umin
 	*(umin+ii)= *(ux+ii);
 	u_lo= *(ux+ii) + 0.000001;
 	u_hi= 1.1 * (*(ux+ii) + 0.000001);
@@ -1632,6 +1628,10 @@ void calcUminUmax(int ndata,
 	*(umax+ii) = gsl_root_fsolver_root ((s+tid)->s);
       }
     }
+    else if ( fabs(peps) < 0.00000001 && fabs(meps) < 0.00000001 && peps <= 0 && meps <= 0 ) {//circular
+	*(umin+ii) = *(ux+ii);
+	*(umax+ii) = *(ux+ii);
+    }
     else {
       u_lo= 0.9 * *(ux+ii);
       u_hi= *(ux+ii);
