@@ -2,13 +2,19 @@ import sys
 import distutils.sysconfig as sysconfig
 import warnings
 import numpy as nu
+from scipy import integrate
 import ctypes
 import ctypes.util
 from numpy.ctypeslib import ndpointer
 import os
 from galpy import potential
 from galpy.util import galpyWarning
+from ..potential.Potential import _evaluateRforces, _evaluatezforces,\
+    _evaluatephiforces
 from .integratePlanarOrbit import _parse_integrator, _parse_tol
+from ..util.multi import parallel_map
+from ..util.leung_dop853 import dop853
+from ..util import bovy_symplecticode as symplecticode
 #Find and load the library
 _lib= None
 outerr= None
@@ -224,7 +230,7 @@ def _parse_pot(pot,potforactions=False,potfortorus=False):
             pot_args.append(wrap_npot)
             pot_type.extend(wrap_pot_type)
             pot_args.extend(wrap_pot_args)
-            pot_args.extend([p._amp,p._tform,p._tsteady])
+            pot_args.extend([p._amp,p._tform,p._tsteady,int(p._grow)])
         elif isinstance(p,potential.SolidBodyRotationWrapperPotential):
             pot_type.append(-2)
             # Not sure how to easily avoid this duplication
@@ -254,6 +260,21 @@ def _parse_pot(pot,potforactions=False,potfortorus=False):
             pot_type.extend(wrap_pot_type)
             pot_args.extend(wrap_pot_args)
             pot_args.extend([p._amp,p._to,p._sigma2])
+        elif isinstance(p,potential.MovingObjectPotential):
+            pot_type.append(-6)
+            wrap_npot, wrap_pot_type, wrap_pot_args= \
+                _parse_pot(p._pot,
+                           potforactions=potforactions,potfortorus=potfortorus)
+            pot_args.append(wrap_npot)
+            pot_type.extend(wrap_pot_type)
+            pot_args.extend(wrap_pot_args)
+            pot_args.extend([len(p._orb.t)])
+            pot_args.extend(p._orb.t)
+            pot_args.extend(p._orb.x(p._orb.t,use_physical=False))
+            pot_args.extend(p._orb.y(p._orb.t,use_physical=False))
+            pot_args.extend(p._orb.z(p._orb.t,use_physical=False))
+            pot_args.extend([p._amp])
+            pot_args.extend([p._orb.t[0],p._orb.t[-1]]) #t_0, t_f
     pot_type= nu.array(pot_type,dtype=nu.int32,order='C')
     pot_args= nu.array(pot_args,dtype=nu.float64,order='C')
     return (npot,pot_type,pot_args)
@@ -277,20 +298,25 @@ def integrateFullOrbit_c(pot,yo,t,int_method,rtol=None,atol=None,dt=None):
        C integrate an ode for a FullOrbit
     INPUT:
        pot - Potential or list of such instances
-       yo - initial condition [q,p]
+       yo - initial condition [q,p] , can be [N,6] or [6]
        t - set of times at which one wants the result
        int_method= 'leapfrog_c', 'rk4_c', 'rk6_c', 'symplec4_c'
        rtol, atol
-       dt= (None) force integrator to use this stepsize (default is to automatically determine one))
+       dt= (None) force integrator to use this stepsize (default is to automatically determine one; only for C-based integrators)
     OUTPUT:
        (y,err)
-       y : array, shape (len(y0), len(t))
+       y : array, shape (N,len(t),6)  or (len(t),6) if N = 1
        Array containing the value of y for each desired time in t, \
        with the initial value y0 in the first row.
        err: error message, if not zero: 1 means maximum step reduction happened for adaptive integrators
     HISTORY:
        2011-11-13 - Written - Bovy (IAS)
+       2018-12-21 - Adapted to allow multiple objects - Bovy (UofT)
     """
+    if len(yo.shape) == 1: single_obj= True
+    else: single_obj= False
+    yo= nu.atleast_2d(yo)
+    nobj= len(yo)
     rtol, atol= _parse_tol(rtol,atol)
     npot, pot_type, pot_args= _parse_pot(pot)
     int_method_c= _parse_integrator(int_method)
@@ -298,13 +324,14 @@ def integrateFullOrbit_c(pot,yo,t,int_method,rtol=None,atol=None,dt=None):
         dt= -9999.99
 
     #Set up result array
-    result= nu.empty((len(t),6))
-    err= ctypes.c_int(0)
+    result= nu.empty((nobj,len(t),6))
+    err= nu.zeros(nobj,dtype=nu.int32)
 
     #Set up the C code
     ndarrayFlags= ('C_CONTIGUOUS','WRITEABLE')
     integrationFunc= _lib.integrateFullOrbit
-    integrationFunc.argtypes= [ndpointer(dtype=nu.float64,flags=ndarrayFlags),
+    integrationFunc.argtypes= [ctypes.c_int,
+                               ndpointer(dtype=nu.float64,flags=ndarrayFlags),
                                ctypes.c_int,                             
                                ndpointer(dtype=nu.float64,flags=ndarrayFlags),
                                ctypes.c_int,
@@ -314,7 +341,7 @@ def integrateFullOrbit_c(pot,yo,t,int_method,rtol=None,atol=None,dt=None):
                                ctypes.c_double,
                                ctypes.c_double,
                                ndpointer(dtype=nu.float64,flags=ndarrayFlags),
-                               ctypes.POINTER(ctypes.c_int),
+                               ndpointer(dtype=nu.int32,flags=ndarrayFlags),
                                ctypes.c_int]
 
     #Array requirements, first store old order
@@ -323,28 +350,32 @@ def integrateFullOrbit_c(pot,yo,t,int_method,rtol=None,atol=None,dt=None):
     yo= nu.require(yo,dtype=nu.float64,requirements=['C','W'])
     t= nu.require(t,dtype=nu.float64,requirements=['C','W'])
     result= nu.require(result,dtype=nu.float64,requirements=['C','W'])
+    err= nu.require(err,dtype=nu.int32,requirements=['C','W'])
 
     #Run the C code
-    integrationFunc(yo,
+    integrationFunc(ctypes.c_int(nobj),
+                    yo,
                     ctypes.c_int(len(t)),
                     t,
                     ctypes.c_int(npot),
                     pot_type,
                     pot_args,
                     ctypes.c_double(dt),
-                    ctypes.c_double(rtol),ctypes.c_double(atol),
+                    ctypes.c_double(rtol),
+                    ctypes.c_double(atol),
                     result,
-                    ctypes.byref(err),
+                    err,
                     ctypes.c_int(int_method_c))
     
-    if int(err.value) == -10: #pragma: no cover
+    if nu.any(err == -10): #pragma: no cover
         raise KeyboardInterrupt("Orbit integration interrupted by CTRL-C (SIGINT)")
 
     #Reset input arrays
     if f_cont[0]: yo= nu.asfortranarray(yo)
     if f_cont[1]: t= nu.asfortranarray(t)
 
-    return (result,err.value)
+    if single_obj: return (result[0],err[0])
+    else: return (result,err)
 
 def integrateFullOrbit_dxdv_c(pot,yo,dyo,t,int_method,rtol=None,atol=None): #pragma: no cover because not included in v1, uncover when included
     """
@@ -419,3 +450,195 @@ def integrateFullOrbit_dxdv_c(pot,yo,dyo,t,int_method,rtol=None,atol=None): #pra
     if f_cont[1]: t= nu.asfortranarray(t)
 
     return (result,err.value)
+
+def integrateFullOrbit(pot,yo,t,int_method,rtol=None,atol=None,numcores=1,
+                       dt=None):
+    """
+    NAME:
+       integrateFullOrbit
+    PURPOSE:
+       Integrate an ode for a FullOrbit
+    INPUT:
+       pot - Potential or list of such instances
+       yo - initial condition [q,p], shape [N,5] or [N,6]
+       t - set of times at which one wants the result
+       int_method= 'leapfrog', 'odeint', or 'dop853'
+       rtol, atol= tolerances (not always used...)
+       numcores= (1) number of cores to use for multi-processing
+       dt= (None) force integrator to use this stepsize (default is to automatically determine one; only for C-based integrators)
+    OUTPUT:
+       (y,err)
+       y : array, shape (N,len(t),5/6)
+       Array containing the value of y for each desired time in t, \
+       with the initial value y0 in the first row.
+       err: error message, always zero for now
+    HISTORY:
+       2010-08-01 - Written - Bovy (NYU)
+       2019-04-09 - Adapted to allow multiple objects and parallel mapping - Bovy (UofT)
+    """
+    nophi= False
+    if not int_method.lower() == 'dop853' and not int_method == 'odeint':
+        if len(yo[0]) == 5:
+            nophi= True
+            #We hack this by putting in a dummy phi=0
+            yo= nu.pad(yo,((0,0),(0,1)),'constant',constant_values=0)
+    if int_method.lower() == 'leapfrog':
+        if rtol is None: rtol= 1e-8
+        def integrate_for_map(vxvv):
+            #go to the rectangular frame
+            this_vxvv= nu.array([vxvv[0]*nu.cos(vxvv[5]),
+                                 vxvv[0]*nu.sin(vxvv[5]),
+                                 vxvv[3],
+                                 vxvv[1]*nu.cos(vxvv[5])
+                                     -vxvv[2]*nu.sin(vxvv[5]),
+                                 vxvv[2]*nu.cos(vxvv[5])
+                                     +vxvv[1]*nu.sin(vxvv[5]),
+                                 vxvv[4]])
+            #integrate
+            out= symplecticode.leapfrog(_rectForce,this_vxvv,
+                                        t,args=(pot,),rtol=rtol)
+            #go back to the cylindrical frame
+            R= nu.sqrt(out[:,0]**2.+out[:,1]**2.)
+            phi= nu.arccos(out[:,0]/R)
+            phi[(out[:,1] < 0.)]= 2.*nu.pi-phi[(out[:,1] < 0.)]
+            vR= out[:,3]*nu.cos(phi)+out[:,4]*nu.sin(phi)
+            vT= out[:,4]*nu.cos(phi)-out[:,3]*nu.sin(phi)
+            out[:,3]= out[:,2]
+            out[:,4]= out[:,5]
+            out[:,0]= R
+            out[:,1]= vR
+            out[:,2]= vT
+            out[:,5]= phi
+            return out
+    elif int_method.lower() == 'dop853' or int_method.lower() == 'odeint':
+        if rtol is None: rtol= 1e-8
+        if int_method.lower() == 'dop853':
+            integrator= dop853
+            extra_kwargs= {}
+        else:
+            integrator= integrate.odeint
+            extra_kwargs= {'rtol':rtol}
+        if len(yo[0]) == 5:
+            def integrate_for_map(vxvv):
+                l= vxvv[0]*vxvv[2]
+                l2= l**2.
+                init= [vxvv[0],vxvv[1],vxvv[3],vxvv[4]]
+                intOut= integrator(_RZEOM,init,t=t,args=(pot,l2),
+                                   **extra_kwargs)
+                out= nu.zeros((len(t),5))
+                out[:,0]= intOut[:,0]
+                out[:,1]= intOut[:,1]
+                out[:,3]= intOut[:,2]
+                out[:,4]= intOut[:,3]
+                out[:,2]= l/out[:,0]
+                #post-process to remove negative radii
+                neg_radii= (out[:,0] < 0.)
+                out[neg_radii,0]= -out[neg_radii,0]
+                return out
+        else:
+            def integrate_for_map(vxvv):
+                vphi= vxvv[2]/vxvv[0]
+                init= [vxvv[0],vxvv[1],vxvv[5],vphi,vxvv[3],vxvv[4]]
+                intOut= integrator(_EOM,init,t=t,args=(pot,))
+                out= nu.zeros((len(t),6))
+                out[:,0]= intOut[:,0]
+                out[:,1]= intOut[:,1]
+                out[:,2]= out[:,0]*intOut[:,3]
+                out[:,3]= intOut[:,4]
+                out[:,4]= intOut[:,5]
+                out[:,5]= intOut[:,2]
+                #post-process to remove negative radii
+                neg_radii= (out[:,0] < 0.)
+                out[neg_radii,0]= -out[neg_radii,0]
+                out[neg_radii,3]+= nu.pi
+                return out
+    else: # Assume we are forcing parallel_mapping of a C integrator...
+        def integrate_for_map(vxvv):
+            return integrateFullOrbit_c(pot,nu.copy(vxvv),
+                                        t,int_method,dt=dt)[0]
+    if len(yo) == 1: # Can't map a single value...
+        out= nu.atleast_3d(integrate_for_map(yo[0]).T).T
+    else:
+        out= nu.array((parallel_map(integrate_for_map,yo,numcores=numcores)))
+    if nophi:
+        out= out[:,:,:5]
+    return out, nu.zeros(len(yo))
+
+def _RZEOM(y,t,pot,l2):
+    """
+    NAME:
+       _RZEOM
+    PURPOSE:
+       implements the EOM, i.e., the right-hand side of the differential 
+       equation, for a 3D orbit assuming conservation of angular momentum
+    INPUT:
+       y - current phase-space position
+       t - current time
+       pot - (list of) Potential instance(s)
+       l2 - angular momentum squared
+    OUTPUT:
+       dy/dt
+    HISTORY:
+       2010-04-16 - Written - Bovy (NYU)
+    """
+    return [y[1],
+            l2/y[0]**3.+_evaluateRforces(pot,y[0],y[2],t=t),
+            y[3],
+            _evaluatezforces(pot,y[0],y[2],t=t)]
+
+def _EOM(y,t,pot):
+    """
+    NAME:
+       _EOM
+    PURPOSE:
+       implements the EOM, i.e., the right-hand side of the differential 
+       equation, for a 3D orbit
+    INPUT:
+       y - current phase-space position
+       t - current time
+       pot - (list of) Potential instance(s)
+    OUTPUT:
+       dy/dt
+    HISTORY:
+       2010-04-16 - Written - Bovy (NYU)
+    """
+    l2= (y[0]**2.*y[3])**2.
+    return [y[1],
+            l2/y[0]**3.+_evaluateRforces(pot,y[0],y[4],phi=y[2],t=t,
+                                         v=[y[1],y[0]*y[3],y[5]]),
+            y[3],
+            1./y[0]**2.*(_evaluatephiforces(pot,y[0],y[4],phi=y[2],t=t,
+                                            v=[y[1],y[0]*y[3],y[5]])
+                         -2.*y[0]*y[1]*y[3]),
+            y[5],
+            _evaluatezforces(pot,y[0],y[4],phi=y[2],t=t,
+                             v=[y[1],y[0]*y[3],y[5]])]
+
+def _rectForce(x,pot,t=0.):
+    """
+    NAME:
+       _rectForce
+    PURPOSE:
+       returns the force in the rectangular frame
+    INPUT:
+       x - current position
+       t - current time
+       pot - (list of) Potential instance(s)
+    OUTPUT:
+       force
+    HISTORY:
+       2011-02-02 - Written - Bovy (NYU)
+    """
+    #x is rectangular so calculate R and phi
+    R= nu.sqrt(x[0]**2.+x[1]**2.)
+    phi= nu.arccos(x[0]/R)
+    sinphi= x[1]/R
+    cosphi= x[0]/R
+    if x[1] < 0.: phi= 2.*nu.pi-phi
+    #calculate forces
+    Rforce= _evaluateRforces(pot,R,x[2],phi=phi,t=t)
+    phiforce= _evaluatephiforces(pot,R,x[2],phi=phi,t=t)
+    return nu.array([cosphi*Rforce-1./R*sinphi*phiforce,
+                     sinphi*Rforce+1./R*cosphi*phiforce,
+                     _evaluatezforces(pot,R,x[2],phi=phi,t=t)])
+
