@@ -8,13 +8,21 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <math.h>
+#include <gsl/gsl_errno.h>
+#include <gsl/gsl_spline.h>
+#include <bovy_coords.h>
 #include <bovy_symplecticode.h>
+#include <leung_dop853.h>
 #include <bovy_rk.h>
+#include <integrateFullOrbit.h>
 //Potentials
 #include <galpy_potentials.h>
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
+#ifndef ORBITS_CHUNKSIZE
+#define ORBITS_CHUNKSIZE 1
+#endif
 //Macros to export functions in DLL on different OS
 #if defined(_WIN32)
 #define EXPORT __declspec(dllexport)
@@ -43,6 +51,7 @@ void evalRectDeriv(double, double *, double *,
 			 int, struct potentialArg *);
 void evalRectDeriv_dxdv(double,double *, double *,
 			      int, struct potentialArg *);
+void initMovingObjectSplines(struct potentialArg *, double ** pot_args);
 /*
   Actual functions
 */
@@ -322,13 +331,15 @@ void parse_leapFuncArgs_Full(int npot,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args 
 					    + (int) (*(*pot_args+7) + 20)));
       break;    
+      // 31: KGPotential
+      // 32: IsothermalDiskPotential
 //////////////////////////////// WRAPPERS /////////////////////////////////////
     case -1: //DehnenSmoothWrapperPotential
       potentialArgs->potentialEval= &DehnenSmoothWrapperPotentialEval;
       potentialArgs->Rforce= &DehnenSmoothWrapperPotentialRforce;
       potentialArgs->zforce= &DehnenSmoothWrapperPotentialzforce;
       potentialArgs->phiforce= &DehnenSmoothWrapperPotentialphiforce;
-      potentialArgs->nargs= (int) 3;
+      potentialArgs->nargs= (int) 4;
       break;
     case -2: //SolidBodyRotationWrapperPotential
       potentialArgs->Rforce= &SolidBodyRotationWrapperPotentialRforce;
@@ -349,7 +360,14 @@ void parse_leapFuncArgs_Full(int npot,
       potentialArgs->phiforce= &GaussianAmplitudeWrapperPotentialphiforce;
       potentialArgs->nargs= (int) 3;
       break;
+    case -6: //MovingObjectPotential
+      potentialArgs->Rforce= &MovingObjectPotentialRforce;
+      potentialArgs->zforce= &MovingObjectPotentialzforce;
+      potentialArgs->phiforce= &MovingObjectPotentialphiforce;
+      potentialArgs->nargs= (int) 3;
+      break;
     }
+    int setupSplines = *(*pot_type-1) == -6 ? 1 : 0;
     if ( *(*pot_type-1) < 0 ) { // Parse wrapped potential for wrappers
       potentialArgs->nwrapped= (int) *(*pot_args)++;
       potentialArgs->wrappedPotentialArg= \
@@ -359,6 +377,7 @@ void parse_leapFuncArgs_Full(int npot,
 			      potentialArgs->wrappedPotentialArg,
 			      pot_type,pot_args);
     }
+    if (setupSplines) initMovingObjectSplines(potentialArgs, pot_args);
     potentialArgs->args= (double *) malloc( potentialArgs->nargs * sizeof(double));
     for (jj=0; jj < potentialArgs->nargs; jj++){
       *(potentialArgs->args)= *(*pot_args)++;
@@ -369,7 +388,8 @@ void parse_leapFuncArgs_Full(int npot,
   }
   potentialArgs-= npot;
 }
-EXPORT void integrateFullOrbit(double *yo,
+EXPORT void integrateFullOrbit(int nobj,
+			       double *yo,
 			       int nt, 
 			       double *t,
 			       int npot,
@@ -382,9 +402,21 @@ EXPORT void integrateFullOrbit(double *yo,
 			       int * err,
 			       int odeint_type){
   //Set up the forces, first count
+  int ii,jj;
   int dim;
-  struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  int max_threads;
+  int * thread_pot_type;
+  double * thread_pot_args;
+  max_threads= ( nobj < omp_get_max_threads() ) ? nobj : omp_get_max_threads();
+  // Because potentialArgs may cache, safest to have one / thread
+  struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( max_threads * npot * sizeof (struct potentialArg) );
+#pragma omp parallel for schedule(static,1) private(ii,thread_pot_type,thread_pot_args) num_threads(max_threads) 
+  for (ii=0; ii < max_threads; ii++) {
+    thread_pot_type= pot_type; // need to make thread-private pointers, bc
+    thread_pot_args= pot_args; // these pointers are changed in parse_...
+    parse_leapFuncArgs_Full(npot,potentialArgs+ii*npot,
+			    &thread_pot_type,&thread_pot_args);
+  }
   //Integrate
   void (*odeint_func)(void (*func)(double, double *, double *,
 			   int, struct potentialArg *),
@@ -427,11 +459,25 @@ EXPORT void integrateFullOrbit(double *yo,
     odeint_deriv_func= &evalRectDeriv;
     dim= 6;
     break;
+  case 6: //DOP853
+    odeint_func= &dop853;
+    odeint_deriv_func= &evalRectDeriv;
+    dim= 6;
+    break;
+  }
+#pragma omp parallel for schedule(dynamic,ORBITS_CHUNKSIZE) private(ii,jj) num_threads(max_threads)
+  for (ii=0; ii < nobj; ii++) {
+    cyl_to_rect_galpy(yo+6*ii);
+    odeint_func(odeint_deriv_func,dim,yo+6*ii,nt,dt,t,
+		npot,potentialArgs+omp_get_thread_num()*npot,rtol,atol,
+		result+6*nt*ii,err+ii);
+    for (jj=0; jj < nt; jj++)
+      rect_to_cyl_galpy(result+6*jj+6*nt*ii);
   }
-  odeint_func(odeint_deriv_func,dim,yo,nt,dt,t,npot,potentialArgs,rtol,atol,
-	      result,err);
   //Free allocated memory
-  free_potentialArgs(npot,potentialArgs);
+#pragma omp parallel for schedule(static,1) private(ii) num_threads(max_threads)
+  for (ii=0; ii < max_threads; ii++)
+    free_potentialArgs(npot,potentialArgs+ii*npot);
   free(potentialArgs);
   //Done!
 }
@@ -493,6 +539,11 @@ void integrateOrbit_dxdv(double *yo,
     odeint_deriv_func= &evalRectDeriv_dxdv;
     dim= 12;
     break;
+  case 6: //DOP853
+    odeint_func= &dop853;
+    odeint_deriv_func= &evalRectDeriv_dxdv;
+    dim= 12;
+    break;
   }
   odeint_func(odeint_deriv_func,dim,yo,nt,-9999.99,t,npot,potentialArgs,
 	      rtol,atol,result,err);
@@ -548,6 +599,45 @@ void evalRectDeriv(double t, double *q, double *a,
   *a= zforce;
 }
 
+void initMovingObjectSplines(struct potentialArg * potentialArgs,
+			     double ** pot_args){
+  gsl_interp_accel *x_accel_ptr = gsl_interp_accel_alloc();
+  gsl_interp_accel *y_accel_ptr = gsl_interp_accel_alloc();
+  gsl_interp_accel *z_accel_ptr = gsl_interp_accel_alloc();
+  int nPts = (int) **pot_args;
+
+  gsl_spline *x_spline = gsl_spline_alloc(gsl_interp_cspline, nPts);
+  gsl_spline *y_spline = gsl_spline_alloc(gsl_interp_cspline, nPts);
+  gsl_spline *z_spline = gsl_spline_alloc(gsl_interp_cspline, nPts);
+
+  double * t_arr = *pot_args+1;
+  double * x_arr = t_arr+1*nPts;
+  double * y_arr = t_arr+2*nPts;
+  double * z_arr = t_arr+3*nPts;
+
+  double * t= (double *) malloc ( nPts * sizeof (double) );
+  double tf = *(t_arr+4*nPts+2);
+  double to = *(t_arr+4*nPts+1);
+
+  int ii;
+  for (ii=0; ii < nPts; ii++)
+    *(t+ii) = (t_arr[ii]-to)/(tf-to);
+
+  gsl_spline_init(x_spline, t, x_arr, nPts);
+  gsl_spline_init(y_spline, t, y_arr, nPts);
+  gsl_spline_init(z_spline, t, z_arr, nPts);
+
+  potentialArgs->xSpline = x_spline;
+  potentialArgs->accx = x_accel_ptr;
+  potentialArgs->ySpline = y_spline;
+  potentialArgs->accy = y_accel_ptr;
+  potentialArgs->zSpline = z_spline;
+  potentialArgs->accz = z_accel_ptr;
+
+  *pot_args = *pot_args + (int) (1+4*nPts);
+  free(t);
+}
+
 // LCOV_EXCL_START
 void evalRectDeriv_dxdv(double t, double *q, double *a,
 			int nargs, struct potentialArg * potentialArgs){
