@@ -18,19 +18,45 @@ from __future__  import division, print_function
 import os, os.path
 import pickle
 from functools import wraps
+import warnings
 import math
 import numpy as nu
 from scipy import optimize, integrate
 import galpy.util.bovy_plot as plot
 from galpy.util import bovy_coords
-from galpy.util.bovy_conversion import velocity_in_kpcGyr, \
-    physical_conversion, potential_physical_input, freq_in_Gyr
+from ..util.bovy_conversion import velocity_in_kpcGyr, \
+    physical_conversion, potential_physical_input, freq_in_Gyr, \
+    get_physical
+from ..util import config
+from ..util import galpyWarning
 from .plotRotcurve import plotRotcurve, vcirc
 from .plotEscapecurve import _INF, plotEscapecurve
 from .DissipativeForce import DissipativeForce, _isDissipative
 from .Force import Force, _APY_LOADED
 if _APY_LOADED:
     from astropy import units
+def check_potential_inputs_not_arrays(func):
+    """
+    NAME:
+       check_potential_inputs_not_arrays
+    PURPOSE:
+       Decorator to check inputs and throw TypeError if any of the inputs are arrays for Potentials that do not support array evaluation
+    HISTORY:
+       2017-summer - Written for SpiralArmsPotential - Jack Hong (UBC)
+       2019-05-23 - Moved to Potential for more general use - Bovy (UofT)
+       
+    """
+    @wraps(func)
+    def func_wrapper(self,R,z,phi,t):
+        if (hasattr(R,'shape') and R.shape != () and len(R) > 1) \
+                or (hasattr(z,'shape') and z.shape != () and len(z) > 1) \
+                or (hasattr(phi,'shape') and phi.shape != () and len(phi) > 1) \
+                or (hasattr(t,'shape') and t.shape != () and len(t) > 1):
+            raise TypeError('Methods in {} do not accept array inputs. Please input scalars'.format(self.__class__.__name__))
+        return func(self,R,z,phi,t)
+    return func_wrapper
+
+
 class Potential(Force):
     """Top-level class for a potential"""
     def __init__(self,amp=1.,ro=None,vo=None,amp_units=None):
@@ -368,6 +394,8 @@ class Potential(Force):
 
            2014-01-29 - Written - Bovy (IAS)
 
+           2019-08-15 - Added spherical warning - Bovy (UofT)
+
         """
         if self.isNonAxi:
             raise NotImplementedError('mass for non-axisymmetric potentials is not currently supported')
@@ -377,6 +405,10 @@ class Potential(Force):
         except AttributeError:
             #Use numerical integration to get the mass
             if z is None:
+                warnings.warn("Vertical height z not specified for mass "
+                              "calculation...assuming spherical potential"
+                              " (for the mass of axisymmetric potentials"
+                              ", specify z)",galpyWarning)
                 return 4.*nu.pi\
                     *integrate.quad(lambda x: x**2.\
                                         *self.dens(x,0.,
@@ -650,7 +682,7 @@ class Potential(Force):
             return self._amp*self._phi2deriv(R,Z,phi=phi,t=t)
         except AttributeError: #pragma: no cover
             if self.isNonAxi:
-                raise PotentialError("'_phiforce' function not implemented for this non-axisymmetric potential")
+                raise PotentialError("'_phi2deriv' function not implemented for this non-axisymmetric potential")
             return 0.
 
     @potential_physical_input
@@ -688,7 +720,7 @@ class Potential(Force):
             return self._amp*self._Rphideriv(R,Z,phi=phi,t=t)
         except AttributeError: #pragma: no cover
             if self.isNonAxi:
-                raise PotentialError("'_phiforce' function not implemented for this non-axisymmetric potential")
+                raise PotentialError("'_Rphideriv' function not implemented for this non-axisymmetric potential")
             return 0.
 
     def toPlanar(self):
@@ -717,7 +749,7 @@ class Potential(Force):
         from galpy.potential import toPlanarPotential
         return toPlanarPotential(self)
 
-    def toVertical(self,R):
+    def toVertical(self,R,phi=None,t0=0.):
         """
         NAME:
 
@@ -731,19 +763,21 @@ class Potential(Force):
 
            R - Galactocentric radius at which to create the vertical potential (can be Quantity)
 
+           phi= (None) Galactocentric azimuth at which to create the vertical potential (can be Quantity); required for non-axisymmetric potential
+
+           t0= (0.) time at which to create the vertical potential (can be Quantity)
+
         OUTPUT:
 
-           linear (vertical) potential
+           linear (vertical) potential: Phi(z,phi,t) = Phi(R,z,phi,t)-Phi(R,0.,phi0,t0) where phi0 and t0 are the phi and t inputs
 
         HISTORY
 
            unknown
 
         """
-        if _APY_LOADED and isinstance(R,units.Quantity):
-            R= R.to(units.kpc).value/self._ro
-        from galpy.potential import RZToverticalPotential
-        return RZToverticalPotential(self,R)
+        from galpy.potential import toVerticalPotential
+        return toVerticalPotential(self,R,phi=phi,t0=t0)
 
     def plot(self,t=0.,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
              effective=False,Lz=None,phi=None,xy=False,
@@ -1515,15 +1549,13 @@ class Potential(Force):
         tyx= R2deriv*sinphi*cosphi+Rphideriv*(cos2phi-sin2phi)/R\
             -Rderiv*sinphi*cosphi/R-phi2deriv*sinphi*cosphi/R2\
             +phideriv*(sin2phi-cos2phi)/R2      
-        tzx=Rzderiv*cosphi-Rderiv*cosphi*z/R2-zphideriv*sinphi/R\
-            +phideriv*2.*sinphi*z/R3
+        tzx=Rzderiv*cosphi-zphideriv*sinphi/R
         tyy=R2deriv*sin2phi+Rphideriv*2.*cosphi*sinphi/R+Rderiv*cos2phi/R\
             +phi2deriv*cos2phi/R2-phideriv*2.*sinphi*cosphi/R2
         txy=tyx
-        tzy=Rzderiv*sinphi-Rderiv*sinphi*z/R2+zphideriv*cosphi/R\
-            -phideriv*2.*cosphi*z/R3
-        txz=Rzderiv*cosphi-zphideriv*sinphi/R
-        tyz=Rzderiv*sinphi+zphideriv*cosphi/R
+        tzy=Rzderiv*sinphi+zphideriv*cosphi/R
+        txz=tzx
+        tyz=tzy
         tzz=z2deriv
         tij=-nu.array([[txx,txy,txz],[tyx,tyy,tyz],[tzx,tzy,tzz]])
         if eigenval:
@@ -2865,6 +2897,52 @@ def nemo_accpars(Pot,vo,ro):
     else: #pragma: no cover 
         raise PotentialError("Input to 'nemo_accpars' is neither a Potential-instance or a list of such instances")
     
+def to_amuse(Pot,t=0.,tgalpy=0.,reverse=False,ro=None,vo=None): # pragma: no cover
+    """
+    NAME:
+    
+       to_amuse
+
+    PURPOSE:
+
+       Return an AMUSE representation of a galpy Potential or list of Potentials
+
+    INPUT:
+
+       Pot - Potential instance or list of such instances
+
+       t= (0.) Initial time in AMUSE (can be in internal galpy units or AMUSE units)
+
+       tgalpy= (0.) Initial time in galpy (can be in internal galpy units or AMUSE units); because AMUSE initial times have to be positive, this is useful to set if the initial time in galpy is negative
+
+       reverse= (False) set whether the galpy potential evolves forwards or backwards in time (default: False); because AMUSE can only integrate forward in time, this is useful to integrate backward in time in AMUSE
+
+       ro= (default taken from Pot) length unit in kpc
+
+       vo= (default taken from Pot) velocity unit in km/s       
+
+    OUTPUT:
+
+       AMUSE representation of Pot
+
+    HISTORY:
+
+       2019-08-04 - Written - Bovy (UofT)
+
+       2019-08-12 - Implemented actual function - Webb (UofT)
+
+    """
+    try:
+        from . import amuse
+    except ImportError:
+        raise ImportError("To obtain an AMUSE representation of a galpy potential, you need to have AMUSE installed")
+    Pot= flatten(Pot)
+    if ro is None or vo is None:
+        physical_dict= get_physical(Pot)
+        if ro is None: ro= physical_dict.get('ro')
+        if vo is None: vo= physical_dict.get('vo')
+    return amuse.galpy_profile(Pot,t=t,tgalpy=tgalpy,ro=ro,vo=vo)
+
 def turn_physical_off(Pot):
     """
     NAME:
@@ -2991,7 +3069,7 @@ def _check_c(Pot,dxdv=False):
 
     """
     Pot= flatten(Pot)
-    from galpy.potential import planarPotential
+    from galpy.potential import planarPotential, linearPotential
     if dxdv: hasC_attr= 'hasC_dxdv'
     else: hasC_attr= 'hasC'
     from .WrapperPotential import parentWrapperPotential
@@ -3000,7 +3078,8 @@ def _check_c(Pot,dxdv=False):
                                dtype='bool'))
     elif isinstance(Pot,parentWrapperPotential):
         return bool(Pot.__dict__[hasC_attr]*_check_c(Pot._pot))
-    elif isinstance(Pot,Potential) or isinstance(Pot,planarPotential):
+    elif isinstance(Pot,Potential) or isinstance(Pot,planarPotential) \
+            or isinstance(Pot,linearPotential):
         return Pot.__dict__[hasC_attr]
 
 def _dim(Pot):
@@ -3178,15 +3257,13 @@ def ttensor(Pot,R,z,phi=0.,t=0.,eigenval=False):
         +phi2deriv*sin2phi/R2+phideriv*2.*cosphi*sinphi/R2
     tyx= R2deriv*sinphi*cosphi+Rphideriv*(cos2phi-sin2phi)/R\
         -Rderiv*sinphi*cosphi/R-phi2deriv*sinphi*cosphi/R2+phideriv*(sin2phi-cos2phi)/R2
-    tzx= Rzderiv*cosphi-Rderiv*cosphi*z/R2-zphideriv*sinphi/R\
-        +phideriv*2.*sinphi*z/R3
+    tzx= Rzderiv*cosphi-zphideriv*sinphi/R
     tyy= R2deriv*sin2phi+Rphideriv*2.*cosphi*sinphi/R+Rderiv*cos2phi/R\
         +phi2deriv*cos2phi/R2-phideriv*2.*sinphi*cosphi/R2
     txy=tyx
-    tzy= Rzderiv*sinphi-Rderiv*sinphi*z/R2+zphideriv*cosphi/R\
-        -phideriv*2.*cosphi*z/R3
-    txz= Rzderiv*cosphi-zphideriv*sinphi/R
-    tyz= Rzderiv*sinphi+zphideriv*cosphi/R
+    tzy= Rzderiv*sinphi+zphideriv*cosphi/R
+    txz= tzx
+    tyz= tzy
     tzz=z2deriv
     tij= -nu.array([[txx,txy,txz],[tyx,tyy,tyz],[tzx,tzy,tzz]])
     if eigenval:
