@@ -13,8 +13,10 @@ from scipy.special import gammaln
 
 import hashlib
 
+from .NumericalPotentialDerivativesMixin import \
+    NumericalPotentialDerivativesMixin
 
-class SCFPotential(Potential):
+class SCFPotential(Potential,NumericalPotentialDerivativesMixin):
     """Class that implements the `Hernquist & Ostriker (1992) <http://adsabs.harvard.edu/abs/1992ApJ...386..375H>`_ Self-Consistent-Field-type potential. 
     Note that we divide the amplitude by 2 such that :math:`Acos = \\delta_{0n}\\delta_{0l}\\delta_{0m}` and :math:`Asin = 0` corresponds to :ref:`Galpy's Hernquist Potential <hernquist_potential>`.
 
@@ -79,6 +81,7 @@ class SCFPotential(Potential):
 
            2016-05-13 - Written - Aladdin 
         """        
+        NumericalPotentialDerivativesMixin.__init__(self,{}) # just use default dR etc.
         Potential.__init__(self,amp=amp/2.,ro=ro,vo=vo,amp_units='mass')
         if _APY_LOADED and isinstance(a,units.Quantity): 
             a= a.to(units.kpc).value/self._ro 
@@ -433,7 +436,7 @@ class SCFPotential(Potential):
         li = _cartesian(shape)
 
         for i in range(li.shape[0]):
-            j = nu.split(li[i], li.shape[1])
+            j = tuple(nu.split(li[i], li.shape[1]))
             dPhi_dr,dPhi_dtheta,dPhi_dphi = \
             self._computeforce(R[j][0],z[j][0],phi[j][0])
             force[j] = dr_dx[j][0]*dPhi_dr + dtheta_dx[j][0]*dPhi_dtheta +dPhi_dphi*dphi_dx[j][0]
