@@ -7,6 +7,8 @@
 #      Main included functions:
 #            radec_to_lb
 #            lb_to_radec
+#            radec_to_custom
+#            custom_to_radec
 #            lbd_to_XYZ
 #            XYZ_to_lbd
 #            rectgal_to_sphergal
@@ -15,6 +17,8 @@
 #            vxvyvz_to_vrpmllpmbb
 #            pmrapmdec_to_pmllpmbb
 #            pmllpmbb_to_pmrapmdec
+#            pmrapmdec_to_custom
+#            custom_to_pmrapmdec
 #            cov_pmrapmdec_to_pmllpmbb
 #            cov_dvrpmllbb_to_vxyz
 #            XYZ_to_galcenrect
@@ -120,26 +124,46 @@ def scalarDecorator(func):
         else:
             return result
     return scalar_wrapper
+
+
 def degreeDecorator(inDegrees,outDegrees):
-    """Decorator to transform angles from and to degrees if necessary"""
+    """
+    NAME:
+
+       degreeDecorator
+
+    PURPOSE:
+
+       Decorator to transform angles from and to degrees if necessary
+
+    INPUT:
+
+       inDegrees - list specifiying indices of angle arguments (e.g., [0,1])
+       outDegrees - list, same as inDegrees, but for function return
+
+    HISTORY:
+
+       ____-__-__ - Written - Bovy
+
+       2019-03-02 - speedup - Nathaniel Starkman (UofT)
+
+    """
+    # (modified) old degree decorator
     def wrapper(func):
         @wraps(func)
-        def wrapped(*args,**kwargs):
-            if kwargs.get('degree',False):
-                newargs= ()
-                for ii in range(len(args)):
-                    if ii in inDegrees:
-                        newargs= newargs+(args[ii]*nu.pi/180.,)
-                    else:
-                        newargs= newargs+(args[ii],)
-                args= newargs
-            out= func(*args,**kwargs)
-            if kwargs.get('degree',False):
-                for indx in outDegrees:
-                    out[:,indx]/= nu.pi/180.
+        def wrapped(*args, **kwargs):
+            isdeg = kwargs.get('degree', False)
+            if isdeg:
+                args = [arg * nu.pi / 180 if i in inDegrees else arg
+                        for i, arg in enumerate(args)]
+            out = func(*args, **kwargs)
+            if isdeg:
+                for i in outDegrees:
+                    out[:, i] *= 180. / nu.pi
             return out
         return wrapped
-    return wrapper            
+    return wrapper
+
 
 @scalarDecorator
 @degreeDecorator([0,1],[0,1])
@@ -1005,12 +1029,10 @@ def rect_to_cyl(X,Y,Z):
 
        2010-09-24 - Written - Bovy (NYU)
 
+       2019-06-21 - Changed such that phi in [-pi,pi] - Bovy (UofT)
+
     """
-    R= sc.sqrt(X**2.+Y**2.)
-    phi= sc.arctan2(Y,X)
-    if isinstance(phi,nu.ndarray): phi[phi<0.]+= 2.*nu.pi
-    elif phi < 0.: phi+= 2.*nu.pi
-    return (R,phi,Z)
+    return (sc.sqrt(X**2.+Y**2.),sc.arctan2(Y,X),Z)
 
 def cyl_to_rect(R,phi,Z):
     """
@@ -2154,9 +2176,10 @@ def lambdanu_to_Rz(l,n,ac=5.,Delta=1.):
         else:                         z2        = 0.
     return (nu.sqrt(r2),nu.sqrt(z2))
 
+
 @scalarDecorator
 @degreeDecorator([0,1],[0,1])
-def radec_to_custom(ra,dec,T=None,degree=False,epoch=2000.0):
+def radec_to_custom(ra,dec,T=None,degree=False):
     """
     NAME:
 
@@ -2188,6 +2211,8 @@ def radec_to_custom(ra,dec,T=None,degree=False,epoch=2000.0):
 
        2014-06-14 - Re-written w/ numpy functions for speed and w/ decorators for beauty - Bovy (IAS)
 
+       2019-03-02 - adjusted angle ranges - Nathaniel (UofT)
+
     """
     if T is None: raise ValueError("Must set T= for radec_to_custom")
     #Whether to use degrees and scalar input is handled by decorators
@@ -2195,14 +2220,16 @@ def radec_to_custom(ra,dec,T=None,degree=False,epoch=2000.0):
                    nu.cos(dec)*nu.sin(ra),
                    nu.sin(dec)])
     galXYZ= nu.dot(T,XYZ)
-    b= nu.arcsin(galXYZ[2])
-    l= nu.arctan2(galXYZ[1]/sc.cos(b),galXYZ[0]/sc.cos(b))
+    b= nu.arcsin(galXYZ[2])  # [-pi/2, pi/2]
+    l= nu.arctan2(galXYZ[1], galXYZ[0])
+    l[l<0] += 2 * nu.pi  # fix range to [0, 2 pi]
     out= nu.array([l,b])
-    return out.T   
+    return out.T
+
 
 @scalarDecorator
 @degreeDecorator([2,3],[])
-def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False,epoch=2000.0):
+def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False):
     """
     NAME:
 
@@ -2226,8 +2253,6 @@ def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False,epoch=2000.0):
 
        degree= (False) if True, ra and dec are given in degrees (default=False)
 
-       epoch= (2000.) epoch of ra,dec (right now only 2000.0 and 1950.0 are supported when not using astropy's transformations internally; when internally using astropy's coordinate transformations, epoch can be None for ICRS, 'JXXXX' for FK5, and 'BXXXX' for FK4)
-
     OUTPUT:
 
        (pmphi1 x cos(phi2),pmph2) for vector inputs [:,2]
@@ -2236,14 +2261,12 @@ def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False,epoch=2000.0):
 
        2016-10-24 - Written - Bovy (UofT/CCA)
 
+       2019-03-09 - uses custom_to_radec - Nathaniel Starkman (UofT)
+
     """
-    if T is None: raise ValueError("Must set T= for radec_to_custom")
+    if T is None: raise ValueError("Must set T= for pmrapmdec_to_custom")
     # Need to figure out ra_ngp and dec_ngp for this custom set of sky coords
-    # Should replace this eventually with custom_to_radec
-    customXYZ= nu.dot(T.T,nu.array([0.,0.,1.]))
-    dec_ngp= nu.arcsin(customXYZ[2])
-    ra_ngp= nu.arctan2(customXYZ[1]/sc.cos(dec_ngp),
-                       customXYZ[0]/sc.cos(dec_ngp))
+    ra_ngp, dec_ngp= custom_to_radec(0., nu.pi/2, T=T)
     #Whether to use degrees and scalar input is handled by decorators
     dec[dec == dec_ngp]+= 10.**-16 #deal w/ pole.
     sindec_ngp= nu.sin(dec_ngp)
@@ -2260,6 +2283,83 @@ def pmrapmdec_to_custom(pmra,pmdec,ra,dec,T=None,degree=False,epoch=2000.0):
     return (nu.array([[cosphi,-sinphi],[sinphi,cosphi]]).T\
                 *nu.array([[pmra,pmra],[pmdec,pmdec]]).T).sum(-1)
 
+
+def custom_to_radec(phi1,phi2,T=None,degree=False):
+    """
+    NAME:
+
+        custom_to_radec
+
+    PURPOSE:
+
+       rotate a custom set of sky coordinates (phi1, phi2) to (ra, dec)
+       given the rotation matrix T for (ra, dec) -> (phi1, phi2)
+
+    INPUT:
+
+        phi1 - custom sky coord
+
+        phi2 - custom sky coord
+
+        T - matrix defining the transformation (ra, dec) -> (phi1, phi2)
+
+        degree - default: False. If True, phi1 and phi2 in degrees
+
+    OUTPUT:
+
+        (ra, dec) for vector inputs [:, 2]
+
+    HISTORY:
+
+        2018-10-23 - Written - Nathaniel (UofT)
+    """
+    if T is None: raise ValueError("Must set T= for custom_to_radec")
+    return radec_to_custom(phi1, phi2,
+                           T=nu.transpose(T),  # T.T = inv(T)
+                           degree=degree)
+
+
+def custom_to_pmrapmdec(pmphi1,pmphi2,phi1,phi2,T=None,degree=False):
+    """
+    NAME:
+
+       custom_to_pmrapmdec
+
+    PURPOSE:
+
+       rotate proper motions in a custom set of sky coordinates (phi1,phi2) to ICRS (ra,dec)
+
+    INPUT:
+
+       pmphi1 - proper motion in custom (multplied with cos(phi2)) [mas/yr]
+
+       pmphi2 - proper motion in phi2 [mas/yr]
+
+       phi1 - custom longitude
+
+       phi2 - custom latitude
+
+       T= matrix defining the transformation in cartesian coordinates:
+          new_rect = T dot old_rect
+          where old_rect = [cos(dec)cos(ra), cos(dec)sin(ra), sin(dec)] and similar for new_rect
+
+       degree= (False) if True, phi1 and phi2 are given in degrees (default=False)
+
+    OUTPUT:
+
+       (pmra x cos(dec), dec) for vector inputs [:,2]
+
+    HISTORY:
+
+       2019-03-02 - Written - Nathaniel Starkman (UofT)
+
+    """
+    if T is None: raise ValueError("Must set T= for custom_to_pmrapmdec")
+    return pmrapmdec_to_custom(pmphi1, pmphi2, phi1, phi2,
+                               T=nu.transpose(T),  # T.T = inv(T)
+                               degree=degree)
+
+
 def get_epoch_angles(epoch=2000.0):
     """
     NAME:
