@@ -32,9 +32,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-#include "signal.h"
 #include <bovy_symplecticode.h>
 #include <bovy_rk.h>
+#include "signal.h"
 #define _MAX_STEPCHANGE_POWERTWO 3.
 #define _MIN_STEPCHANGE_POWERTWO -3.
 #define _MAX_STEPREDUCE 10000.
@@ -92,10 +92,14 @@ void bovy_rk4(void (*func)(double t, double *q, double *a,
   //Integrate the system
   double to= *t;
   // Handle KeyboardInterrupt gracefully
+#ifndef _WIN32
   struct sigaction action;
   memset(&action, 0, sizeof(struct sigaction));
   action.sa_handler= handle_sigint;
   sigaction(SIGINT,&action,NULL);
+#else
+    if (SetConsoleCtrlHandler(CtrlHandler, TRUE)) {}
+#endif
   for (ii=0; ii < (nt-1); ii++){
     if ( interrupted ) {
       *err= -10;
@@ -117,8 +121,10 @@ void bovy_rk4(void (*func)(double t, double *q, double *a,
     for (kk=0; kk < dim; kk++) *(yn+kk)= *(yn1+kk);
   }
   // Back to default handler
+#ifndef _WIN32
   action.sa_handler= SIG_DFL;
   sigaction(SIGINT,&action,NULL);
+#endif
   //Free allocated memory
   free(yn);
   free(yn1);
@@ -190,10 +196,14 @@ void bovy_rk6(void (*func)(double t, double *q, double *a,
   //Integrate the system
   double to= *t;
   // Handle KeyboardInterrupt gracefully
+#ifndef _WIN32
   struct sigaction action;
   memset(&action, 0, sizeof(struct sigaction));
   action.sa_handler= handle_sigint;
   sigaction(SIGINT,&action,NULL);
+#else
+    if (SetConsoleCtrlHandler(CtrlHandler, TRUE)) {}
+#endif
   for (ii=0; ii < (nt-1); ii++){
     if ( interrupted ) {
       *err= -10;
@@ -217,8 +227,10 @@ void bovy_rk6(void (*func)(double t, double *q, double *a,
     for (kk=0; kk < dim; kk++) *(yn+kk)= *(yn1+kk);
   }
   // Back to default handler
+#ifndef _WIN32
   action.sa_handler= SIG_DFL;
   sigaction(SIGINT,&action,NULL);
+#endif
   //Free allocated memory
   free(yn);
   free(yn1);
@@ -267,14 +279,14 @@ void bovy_rk6_onestep(void (*func)(double t, double *q, double *a,
   func(tn+2.*dt/3.,ynk,a,nargs,potentialArgs);
   for (ii=0; ii < dim; ii++) *(yn1+ii) += 81. * dt * *(a+ii) / 120.;
   for (ii=0; ii < dim; ii++) *(k3+ii)= dt * *(a+ii);
-  for (ii=0; ii < dim; ii++) *(ynk+ii)= *(yn+ii) + ( *(k1+ii) 
+  for (ii=0; ii < dim; ii++) *(ynk+ii)= *(yn+ii) + ( *(k1+ii)
 						     + 4. * *(k2+ii)
 						     - *(k3+ii))/12.;
   //calculate k4
   func(tn+dt/3.,ynk,a,nargs,potentialArgs);
   for (ii=0; ii < dim; ii++) *(yn1+ii) += 81.* dt * *(a+ii) / 120.;
   for (ii=0; ii < dim; ii++) *(k4+ii)= dt * *(a+ii);
-  for (ii=0; ii < dim; ii++) *(ynk+ii)= *(yn+ii) + ( -*(k1+ii) 
+  for (ii=0; ii < dim; ii++) *(ynk+ii)= *(yn+ii) + ( -*(k1+ii)
 						     + 18. * *(k2+ii)
 						     - 3. * *(k3+ii)
 						     -6.* *(k4+ii))/16.;
@@ -297,7 +309,7 @@ void bovy_rk6_onestep(void (*func)(double t, double *q, double *a,
 						     -64. * *(k5+ii))/44.;
   //calculate k7
   func(tn+dt,ynk,a,nargs,potentialArgs);
-  for (ii=0; ii < dim; ii++) *(yn1+ii) += 11.* dt * *(a+ii) / 120.;  
+  for (ii=0; ii < dim; ii++) *(yn1+ii) += 11.* dt * *(a+ii) / 120.;
   //yn1 is new value
 }
 
@@ -350,10 +362,10 @@ double rk4_estimate_step(void (*func)(double t, double *y, double *a,int nargs,
       err+= exp(2.*log(fabs(*(y1+ii)-*(y2+ii)))-2.* *(scale+ii));
     }
     err= sqrt(err/dim);
-    if ( ceil(pow(err,1./5.)) > 1. 
+    if ( ceil(pow(err,1./5.)) > 1.
 	 && init_dt / dt * ceil(pow(err,1./5.)) < _MAX_DT_REDUCE)
       dt/= ceil(pow(err,1./5.));
-    else 
+    else
       break;
   }
   //free what we allocated
@@ -368,7 +380,7 @@ double rk4_estimate_step(void (*func)(double t, double *y, double *a,int nargs,
   //printf("%f\n",dt);
   //fflush(stdout);
   return dt;
-} 
+}
 double rk6_estimate_step(void (*func)(double t, double *y, double *a,int nargs, struct potentialArg *),
 			 int dim, double *yo,
 			 double dt, double *t,
@@ -426,10 +438,10 @@ double rk6_estimate_step(void (*func)(double t, double *y, double *a,int nargs,
       err+= exp(2.*log(fabs(*(y1+ii)-*(y2+ii)))-2.* *(scale+ii));
     }
     err= sqrt(err/dim);
-    if ( ceil(pow(err,1./7.)) > 1. 
+    if ( ceil(pow(err,1./7.)) > 1.
 	 && init_dt / dt * ceil(pow(err,1./7.)) < _MAX_DT_REDUCE)
       dt/= ceil(pow(err,1./7.));
-    else 
+    else
       break;
   }
   //free what we allocated
@@ -449,7 +461,7 @@ double rk6_estimate_step(void (*func)(double t, double *y, double *a,int nargs,
   //printf("%f\n",dt);
   //fflush(stdout);
   return dt;
-} 
+}
 /*
 Runge-Kutta Dormand-Prince 5/4 integrator
 Usage:
@@ -510,10 +522,14 @@ void bovy_dopr54(void (*func)(double t, double *q, double *a,
   //set up a1
   func(to,yn,a1,nargs,potentialArgs);
   // Handle KeyboardInterrupt gracefully
+#ifndef _WIN32
   struct sigaction action;
   memset(&action, 0, sizeof(struct sigaction));
   action.sa_handler= handle_sigint;
   sigaction(SIGINT,&action,NULL);
+#else
+    if (SetConsoleCtrlHandler(CtrlHandler, TRUE)) {}
+#endif
   for (ii=0; ii < (nt-1); ii++){
     if ( interrupted ) {
       *err= -10;
@@ -528,8 +544,10 @@ void bovy_dopr54(void (*func)(double t, double *q, double *a,
     result+= dim;
   }
   // Back to default handler
+#ifndef _WIN32
   action.sa_handler= SIG_DFL;
   sigaction(SIGINT,&action,NULL);
+#endif
   // Free allocated memory
   free(a);
   free(a1);
